#!/usr/bin/env python3
# A simple class to get geographical data from fas and FedoraAsk 
# Alberto Rodriguez Sanchez bt0dotninja@fedoraproject.org
# Renato Silva resilva87@fedoraproject.org


from fedora.client.fas2 import AccountSystem
from fedora.client import AuthError
from collections import Counter
from datetime import datetime

import sys
import numpy as np
import pandas as pd
import pygal
import json
import requests
import configparser


class geofp(object):
    '''
    An object for querying the Fedora Account System and get Geographical data from groups and the whole project 
    '''
    def __init__(self,ConfigFile):
        '''A Simple constructor for geofp object'''
        Config = configparser.ConfigParser()
        Config.read(ConfigFile)
        options = Config.options('FAS')
        userdata={}
        for opt in options:
            userdata[opt] = Config.get('FAS', opt)
        
        user=userdata['user']
        password=userdata['pass']
        self.members=None
        self.fas = AccountSystem(username=user, password=password)
    
    def by_group(self, group, active_date=None):
        '''get information by group'''
        groupMembers=self.fas.group_members(group)
        GeoData={}
        for member in groupMembers:
            gd=self.fas.people_by_key(key=u'username', search=member['username'], fields=['status','country_code','latitude','longitude','last_seen'])
            GeoData.update(gd)
        
        #To pandas DataFrame
        self.members=pd.DataFrame.from_dict(GeoData, orient='index')
        self.members['last_seen']=pd.to_datetime(self.members.last_seen)
        #subsetting by date
        if active_date!=None:
            self.members=self.members[(self.members.last_seen >= active_date) & (self.members.status == 'active')]
        return self.members

    def all_fedorians(self,active_date=None):
        '''Crazy method to get all fedorians'''
        people=self.fas.people_by_key(fields=['status','country_code','latitude','longitude','last_seen'])
        self.members=pd.DataFrame.from_dict(people, orient='index')
        self.members['last_seen']=pd.to_datetime(self.members.last_seen)
        #subsetting by date
        if active_date!=None:
            self.members=self.members[(self.members.last_seen >= active_date) & (self.members.status == 'active')]
        return self.members
 
    def ask_byCountry(self,since=None):

        print("Go for coffee...")
        #get all the msg since "since variable" to today
        baseurl = "https://apps.fedoraproject.org/datagrepper/raw?topic=org.fedoraproject.prod.askbot.post.edit"
        users=[]
        if since!=None:
            dlt= (datetime.now() - since).total_seconds()
            params={'rows_per_page':100,'delta':dlt,'size':'small', 'page':1}
        else:
            params={'page':1,'rows_per_page':100, 'size':'small'}
        
        results= requests.get(baseurl,params=params).json()
        for i in range(results['pages']):
            for msg in results['raw_messages']: 
                users.append(msg['msg']['agent']) 
            params['page']=i+1
            rst= requests.get(baseurl,params=params)
            if rst == None:
                break
            else:
                results= rst.json()
        
        #Ask to fas for all the users 
        GeoData={}
        Members=Counter(users)
        for member in Members.keys():
            gd=self.fas.people_by_key(key=u'username', search=member, fields=['status','country_code','latitude','longitude','last_seen'])
            GeoData.update(gd)
        
        #To pandas DataFrame
        self.members=pd.DataFrame.from_dict(GeoData, orient='index')
        self.members['last_seen']=pd.to_datetime(self.members.last_seen)
        self.members['ask_updates']=pd.Series(Members) #experimental
        return self.members

    def plot_world(self,title='Map', label='# Elements', filename='base_file.svg', save=True):
        '''plot a fancy map with heat information '''
        ct = Counter(self.members.country_code.values.tolist())
        data=dict((k.lower(), v) for k,v in ct.items() if k!=None )
        worldmap = pygal.maps.world.World()
        worldmap.title = title
        worldmap.add(label, data)
        if save:
            worldmap.render_to_file(filename)
        else:
            worldmap.render()
            
    def plot_heat(self,title):
        pass
